@@ -28,6 +28,7 @@
 #include "revision.hpp"
 
 #include <cif++/pdb/io.hpp>
+#include <cif++/dictionary_parser.hpp>
 
 #include <exception>
 #include <filesystem>
@@ -48,7 +49,7 @@ std::string ResidueToDSSPLine(const dssp::residue_info &info)
 	// auto& residue = info.residue();
 	auto &residue = info;
 
-	if (residue.asym_id().length() > 1)
+	if (residue.pdb_strand_id().length() > 1)
 		throw std::runtime_error("This file contains data that won't fit in the original DSSP format");
 
 	char code = residue.compound_letter();
@@ -91,7 +92,7 @@ std::string ResidueToDSSPLine(const dssp::residue_info &info)
 	if (info.bend())
 		bend = 'S';
 
-	double alpha = residue.alpha();
+	double alpha = residue.alpha().value_or(360);
 	char chirality = alpha == 360 ? ' ' : (alpha < 0 ? '-' : '+');
 
 	uint32_t bp[2] = {};
@@ -139,7 +140,11 @@ std::string ResidueToDSSPLine(const dssp::residue_info &info)
 		ss, helix[3], helix[0], helix[1], helix[2], bend, chirality, bridgelabel[0], bridgelabel[1],
 		bp[0], bp[1], sheet, floor(info.accessibility() + 0.5),
 		NHO[0], ONH[0], NHO[1], ONH[1],
-		residue.tco(), residue.kappa(), alpha, residue.phi(), residue.psi(),
+		residue.tco().value_or(0),
+		residue.kappa().value_or(360),
+		residue.alpha().value_or(360),
+		residue.phi().value_or(360),
+		residue.psi().value_or(360),
 		cax, cay, caz)
 	    .str();
 }
@@ -153,7 +158,11 @@ void writeDSSP(const dssp &dssp, std::ostream &os)
 	std::time_t today = system_clock::to_time_t(system_clock::now());
 	std::tm *tm = std::gmtime(&today);
 
-	os << "==== Secondary Structure Definition by the program DSSP, NKI version 4.0                           ==== DATE=" << std::put_time(tm, "%F") << "        ." << std::endl
+	std::string version = kVersionNumber;
+	if (version.length() < 10)
+		version.insert(version.end(), 10 - version.length(), ' ');
+
+	os << "==== Secondary Structure Definition by the program DSSP, NKI version " << version << "                    ==== DATE=" << std::put_time(tm, "%F") << "        ." << std::endl
 	   << "REFERENCE W. KABSCH AND C.SANDER, BIOPOLYMERS 22 (1983) 2577-2637                                                              ." << std::endl
 	   << dssp.get_pdb_header_line(dssp::pdb_record_type::HEADER) << '.' << std::endl
 	   << dssp.get_pdb_header_line(dssp::pdb_record_type::COMPND) << '.' << std::endl
@@ -220,8 +229,6 @@ void writeDSSP(const dssp &dssp, std::ostream &os)
 
 void writeBridgePairs(cif::datablock &db, const dssp &dssp)
 {
-	using ResidueInfo = dssp::residue_info;
-
 	auto &hb = db["dssp_struct_bridge_pairs"];
 
 	hb.add_column("id");
@@ -244,23 +251,16 @@ void writeBridgePairs(cif::datablock &db, const dssp &dssp)
 
 	for (auto &res : dssp)
 	{
-		auto write_res = [&](const std::string &prefix, ResidueInfo const &r, double energy)
-		{
-			hb.back().assign({ { prefix + "label_comp_id", r.compound_id() },
-				{ prefix + "label_seq_id", r.seq_id() },
-				{ prefix + "label_asym_id", r.asym_id() },
-				// { prefix + "auth_comp_id", r.compound_id() },
-				{ prefix + "auth_seq_id", r.auth_seq_id() },
-				{ prefix + "auth_asym_id", r.auth_asym_id() },
-				{ prefix + "pdbx_PDB_ins_code", r.pdb_ins_code() } });
-
-			if (not prefix.empty())
-				hb.back().assign({ { prefix + "energy", energy, 1 } });
-		};
-
-		hb.emplace({ { "id", hb.get_unique_id("") } });
-
-		write_res("", res, 0);
+		cif::row_initializer data({
+			{ "id", hb.get_unique_id("") },
+			{ "label_comp_id", res.compound_id() },
+			{ "label_seq_id", res.seq_id() },
+			{ "label_asym_id", res.asym_id() },
+			// { "auth_comp_id", res.compound_id() },
+			{ "auth_seq_id", res.auth_seq_id() },
+			{ "auth_asym_id", res.auth_asym_id() },
+			{ "pdbx_PDB_ins_code", res.pdb_ins_code() }
+		});
 
 		for (int i : { 0, 1 })
 		{
@@ -268,18 +268,67 @@ void writeBridgePairs(cif::datablock &db, const dssp &dssp)
 			const auto &&[donor, donorEnergy] = res.donor(i);
 
 			if (acceptor)
-				write_res(i ? "acceptor_2_" : "acceptor_1_", acceptor, acceptorEnergy);
+			{
+				if (i == 0)
+				{
+					data.emplace_back("acceptor_1_label_comp_id", acceptor.compound_id());
+					data.emplace_back("acceptor_1_label_seq_id", acceptor.seq_id());
+					data.emplace_back("acceptor_1_label_asym_id", acceptor.asym_id());
+					// data.emplace_back("acceptor_1_auth_comp_id", acceptor.compound_id());
+					data.emplace_back("acceptor_1_auth_seq_id", acceptor.auth_seq_id());
+					data.emplace_back("acceptor_1_auth_asym_id", acceptor.auth_asym_id());
+					data.emplace_back("acceptor_1_pdbx_PDB_ins_code", acceptor.pdb_ins_code());
+					data.emplace_back("acceptor_1_energy", acceptorEnergy, 1);
+				}
+				else
+				{
+					data.emplace_back("acceptor_2_label_comp_id", acceptor.compound_id());
+					data.emplace_back("acceptor_2_label_seq_id", acceptor.seq_id());
+					data.emplace_back("acceptor_2_label_asym_id", acceptor.asym_id());
+					// data.emplace_back("acceptor_2_auth_comp_id", acceptor.compound_id());
+					data.emplace_back("acceptor_2_auth_seq_id", acceptor.auth_seq_id());
+					data.emplace_back("acceptor_2_auth_asym_id", acceptor.auth_asym_id());
+					data.emplace_back("acceptor_2_pdbx_PDB_ins_code", acceptor.pdb_ins_code());
+					data.emplace_back("acceptor_2_energy", acceptorEnergy, 1);
+				}
+			}
 
 			if (donor)
-				write_res(i ? "donor_2_" : "donor_1_", donor, donorEnergy);
+			{
+				if (i == 0)
+				{
+					data.emplace_back("donor_1_label_comp_id", donor.compound_id());
+					data.emplace_back("donor_1_label_seq_id", donor.seq_id());
+					data.emplace_back("donor_1_label_asym_id", donor.asym_id());
+					// data.emplace_back("donor_1_auth_comp_id", donor.compound_id());
+					data.emplace_back("donor_1_auth_seq_id", donor.auth_seq_id());
+					data.emplace_back("donor_1_auth_asym_id", donor.auth_asym_id());
+					data.emplace_back("donor_1_pdbx_PDB_ins_code", donor.pdb_ins_code());
+					data.emplace_back("donor_1_energy", donorEnergy, 1);
+				}
+				else
+				{
+					data.emplace_back("donor_2_label_comp_id", donor.compound_id());
+					data.emplace_back("donor_2_label_seq_id", donor.seq_id());
+					data.emplace_back("donor_2_label_asym_id", donor.asym_id());
+					// data.emplace_back("donor_2_auth_comp_id", donor.compound_id());
+					data.emplace_back("donor_2_auth_seq_id", donor.auth_seq_id());
+					data.emplace_back("donor_2_auth_asym_id", donor.auth_asym_id());
+					data.emplace_back("donor_2_pdbx_PDB_ins_code", donor.pdb_ins_code());
+					data.emplace_back("donor_2_energy", donorEnergy, 1);
+				}
+			}
 		}
+
+		hb.emplace(std::move(data));
 	}
 }
 
 void writeSheets(cif::datablock &db, const dssp &dssp)
 {
+	using namespace cif::literals;
+
 	using res_list = std::vector<dssp::residue_info>;
-	using ss_type = dssp::structure_type;
 
 	// clean up old info first
 
@@ -290,427 +339,44 @@ void writeSheets(cif::datablock &db, const dssp &dssp)
 
 	// create a list of strands, based on the SS info in DSSP. Store sheet number along with the strand.
 
-	std::vector<std::tuple<int, res_list>> strands;
-	std::map<std::tuple<std::string, int>, int> sheetMap; // mapping from bridge number (=info.sheet()) in DSSP to sheet ID
-
-	ss_type ss = ss_type::Loop;
+	std::map<std::tuple<int,int>, res_list> strands;
+	std::set<int> sheetNrs;
 
 	for (auto &res : dssp)
 	{
-		std::tuple<std::string, int> sheetID{ res.asym_id(), res.sheet() };
-		ss_type iss = res.type();
-
-		if (iss == ss and ss == ss_type::Strand and sheetMap[sheetID] == std::get<0>(strands.back()))
-		{
-			std::get<1>(strands.back()).emplace_back(res);
-			continue;
-		}
-
-		ss = iss;
-
-		if (ss != ss_type::Strand)
+		if (res.type() != dssp::structure_type::Strand and res.type() != dssp::structure_type::Betabridge)
 			continue;
 
-		if (not sheetMap.count(sheetID))
-			sheetMap[sheetID] = static_cast<int>(sheetMap.size());
-
-		strands.emplace_back(std::make_tuple(sheetMap[sheetID], res_list{ res }));
+		strands[{res.sheet(), res.strand()}].emplace_back(res);
+		sheetNrs.insert(res.sheet());
 	}
 
-	// sort the strands vector
-	std::sort(strands.begin(), strands.end(), [](auto &a, auto &b)
-		{
-		const auto &[sheetA, strandsA] = a;
-		const auto &[sheetB, strandsB] = b;
+	// --------------------------------------------------------------------
 
-		int d = sheetA - sheetB;
-		if (d == 0)
-			d = strandsA.front().nr() - strandsB.front().nr();
-		return d < 0; });
-
-	// write out the struct_sheet, since all info is available now
 	auto &struct_sheet = db["struct_sheet"];
-
-	int lastSheet = -1;
-	for (const auto &[sheetNr, strand] : strands)
-	{
-		if (sheetNr != lastSheet)
-		{
-			struct_sheet.emplace({ { "id", cif::cif_id_for_number(sheetNr) },
-				{ "number_strands", std::count_if(strands.begin(), strands.end(), [nr = sheetNr](std::tuple<int, res_list> const &s)
-										{ return std::get<0>(s) == nr; }) } });
-
-			lastSheet = sheetNr;
-		}
-	}
-
-	// Each residue resides in a single strand which is part of a single sheet
-	// this function returns the sequence number inside the sheet for the strand
-	// containing res
-	auto strandNrForResidue = [&strands, &sheetMap](dssp::residue_info const &res)
-	{
-		for (const auto &[k, iSheet] : sheetMap)
-		{
-			int result = 0;
-			for (auto &&[sheet, strand] : strands)
-			{
-				if (sheet != iSheet)
-					continue;
-
-				if (std::find(strand.begin(), strand.end(), res) != strand.end())
-					return result;
-
-				++result;
-			}
-		}
-
-		assert(false);
-		return -1;
-	};
-
-	// This map is used to record the sense of a ladder, and can be used
-	// to detect ladders already seen.
-	std::map<std::tuple<int, int, int>, std::tuple<int, bool>> ladderSense;
-
-	for (auto &res : dssp)
-	{
-		if (res.type() != ss_type::Strand)
-			continue;
-
-		int s1 = strandNrForResidue(res);
-
-		for (int i : { 0, 1 })
-		{
-			const auto &[p, ladder, parallel] = res.bridge_partner(i);
-			if (not p or p.asym_id() != res.asym_id() or p.sheet() != res.sheet() or p.type() != ss_type::Strand)
-				continue;
-
-			int s2 = strandNrForResidue(p);
-			// assert(s1 != s2);
-			if (s2 == s1)
-				continue;
-
-			std::tuple<std::string, int> sheetID{ res.asym_id(), res.sheet() };
-			int sheet = sheetMap[sheetID];
-
-			auto k = s1 > s2 ? std::make_tuple(sheet, s2, s1) : std::make_tuple(sheet, s1, s2);
-			if (ladderSense.count(k))
-			{
-				// assert(ladderSense[k] == std::make_tuple(ladder, parallel));
-				assert(std::get<1>(ladderSense[k]) == parallel);
-				continue;
-			}
-
-			ladderSense.emplace(k, std::make_tuple(ladder, parallel));
-		}
-	}
-
-	auto &struct_sheet_order = db["struct_sheet_order"];
-	auto &struct_sheet_hbond = db["struct_sheet_hbond"];
-	auto &pdbx_struct_sheet_hbond = db["pdbx_struct_sheet_hbond"];
-	auto &pdbx_poly_seq_scheme = db["pdbx_poly_seq_scheme"];
-
-	for (const auto &[key, value] : ladderSense)
-	{
-		const auto &[sheet, s1, s2] = key;
-		const auto &[ladder, parallel] = value;
-
-		struct_sheet_order.emplace({ { "sheet_id", cif::cif_id_for_number(sheet) },
-			// { "dssp_struct_ladder_id", cif::cifIdForNumber(ladder) },
-			{ "range_id_1", s1 + 1 },
-			{ "range_id_2", s2 + 1 },
-			{ "sense", parallel ? "parallel" : "anti-parallel" } });
-
-		res_list strand1, strand2;
-
-		int strandIx = 0;
-		for (auto const &s : strands)
-		{
-			const auto &[sSheet, strand] = s;
-			if (sSheet != sheet)
-				continue;
-
-			if (strandIx == s1)
-				strand1 = strand;
-			else if (strandIx == s2)
-			{
-				strand2 = strand;
-				break;
-			}
-
-			++strandIx;
-		}
-
-		assert(not(strand1.empty() or strand2.empty()));
-
-		int beg1SeqID = 0, beg2SeqID = 0, end1SeqID = 0, end2SeqID = 0;
-		std::string beg1AtomID, beg2AtomID, end1AtomID, end2AtomID;
-
-		if (parallel)
-		{
-			// I.	a	d	II.	a	d		parallel
-			//		  \			  /
-			//		b	e		b	e								<= the residues forming the bridge
-			// 		  /			  \                      ..
-			//		c	f		c	f
-
-			for (auto b : strand1)
-			{
-				for (int i : { 0, 1 })
-				{
-					const auto &[e, ladder1, parallel1] = b.bridge_partner(i);
-					auto esi = std::find(strand2.begin(), strand2.end(), e);
-
-					if (esi == strand2.end())
-						continue;
-
-					auto bi = std::find(dssp.begin(), dssp.end(), b);
-					assert(bi != dssp.end() and bi != dssp.begin());
-
-					auto a = *std::prev(bi);
-					auto c = *std::next(bi);
-
-					auto ei = std::find(dssp.begin(), dssp.end(), e);
-					assert(ei != dssp.end() and ei != dssp.begin());
-					auto d = *std::prev(ei);
-					auto f = *std::next(ei);
-
-					if (test_bond(e, a) and test_bond(c, e)) // case I.
-					{
-						beg1SeqID = a.seq_id();
-						beg2SeqID = e.seq_id();
-
-						beg1AtomID = "O";
-						beg2AtomID = "N";
-					}
-					else if (test_bond(b, d) and test_bond(f, b)) // case II.
-					{
-						beg1SeqID = b.seq_id();
-						beg2SeqID = d.seq_id();
-
-						beg1AtomID = "N";
-						beg2AtomID = "O";
-					}
-
-					break;
-				}
-
-				if (beg1SeqID)
-					break;
-			}
-
-			std::reverse(strand1.begin(), strand1.end());
-			std::reverse(strand2.begin(), strand2.end());
-
-			for (auto b : strand1)
-			{
-				for (int i : { 0, 1 })
-				{
-					const auto &[e, ladder1, parallel1] = b.bridge_partner(i);
-					auto esi = std::find(strand2.begin(), strand2.end(), e);
-
-					if (esi == strand2.end())
-						continue;
-
-					auto bi = std::find(dssp.begin(), dssp.end(), b);
-					assert(bi != dssp.end() and bi != dssp.begin());
-
-					auto a = *std::next(bi);
-					auto c = *std::prev(bi);
-
-					auto ei = std::find(dssp.begin(), dssp.end(), e);
-					assert(ei != dssp.end() and ei != dssp.begin());
-					auto d = *std::next(ei);
-					auto f = *std::prev(ei);
-
-					if (test_bond(a, e) and test_bond(e, c)) // case I.
-					{
-						end1SeqID = a.seq_id();
-						end2SeqID = e.seq_id();
-
-						end1AtomID = "N";
-						end2AtomID = "O";
-					}
-					else if (test_bond(d, b) and test_bond(b, f)) // case II.
-					{
-						end1SeqID = b.seq_id();
-						end2SeqID = d.seq_id();
-
-						end1AtomID = "O";
-						end2AtomID = "N";
-					}
-
-					break;
-				}
-
-				if (end1SeqID)
-					break;
-			}
-		}
-		else
-		{
-			// III.	a <- f	IV. a	  f		antiparallel
-			//
-			//		b	 e      b <-> e							<= the residues forming the bridge
-			//
-			//		c -> d		c     d
-
-			std::reverse(strand2.begin(), strand2.end());
-
-			for (auto b : strand1)
-			{
-				for (int i : { 0, 1 })
-				{
-					const auto &[e, ladder1, parallel1] = b.bridge_partner(i);
-					auto esi = std::find(strand2.begin(), strand2.end(), e);
-
-					if (esi == strand2.end())
-						continue;
-
-					auto bi = std::find(dssp.begin(), dssp.end(), b);
-					assert(bi != dssp.end() and bi != dssp.begin());
-
-					auto a = *std::prev(bi);
-					auto c = *std::next(bi);
-
-					auto ei = std::find(dssp.begin(), dssp.end(), e);
-					assert(ei != dssp.end() and ei != dssp.begin());
-					auto d = *std::prev(ei);
-					auto f = *std::next(ei);
-
-					if (test_bond(f, a) and test_bond(c, d)) // case III.
-					{
-						beg1SeqID = a.seq_id();
-						beg2SeqID = f.seq_id();
-
-						beg1AtomID = "O";
-						beg2AtomID = "N";
-					}
-					else if (test_bond(b, e) and test_bond(e, b)) // case IV.
-					{
-						beg1SeqID = b.seq_id();
-						beg2SeqID = e.seq_id();
-
-						beg1AtomID = "N";
-						beg2AtomID = "O";
-					}
-
-					break;
-				}
-
-				if (beg1SeqID)
-					break;
-			}
-
-			std::reverse(strand1.begin(), strand1.end());
-			std::reverse(strand2.begin(), strand2.end());
-
-			for (auto b : strand1)
-			{
-				for (int i : { 0, 1 })
-				{
-					const auto &[e, ladder1, parallel1] = b.bridge_partner(i);
-					auto esi = std::find(strand2.begin(), strand2.end(), e);
-
-					if (esi == strand2.end())
-						continue;
-
-					auto bi = std::find(dssp.begin(), dssp.end(), b);
-					assert(bi != dssp.end() and bi != dssp.begin());
-
-					auto a = *std::next(bi);
-					auto c = *std::prev(bi);
-
-					auto ei = std::find(dssp.begin(), dssp.end(), e);
-					assert(ei != dssp.end() and ei != dssp.begin());
-					auto d = *std::next(ei);
-					auto f = *std::prev(ei);
-
-					if (test_bond(a, f) and test_bond(d, c)) // case III.
-					{
-						end1SeqID = a.seq_id();
-						end2SeqID = f.seq_id();
-
-						end1AtomID = "N";
-						end2AtomID = "O";
-					}
-					else if (test_bond(b, e) and test_bond(e, b)) // case IV.
-					{
-						end1SeqID = b.seq_id();
-						end2SeqID = e.seq_id();
-
-						end1AtomID = "N";
-						end2AtomID = "O";
-					}
-
-					break;
-				}
-
-				if (end1SeqID)
-					break;
-			}
-		}
-
-		struct_sheet_hbond.emplace({ { "sheet_id", cif::cif_id_for_number(sheet) },
-			{ "range_id_1", s1 + 1 },
-			{ "range_id_2", s2 + 1 },
-			{ "range_1_beg_label_seq_id", beg1SeqID },
-			{ "range_1_beg_label_atom_id", beg1AtomID },
-			{ "range_2_beg_label_seq_id", beg2SeqID },
-			{ "range_2_beg_label_atom_id", beg2AtomID },
-			{ "range_1_end_label_seq_id", end1SeqID },
-			{ "range_1_end_label_atom_id", end1AtomID },
-			{ "range_2_end_label_seq_id", end2SeqID },
-			{ "range_2_end_label_atom_id", end2AtomID } });
-
-		using namespace cif::literals;
-
-		auto b1 = pdbx_poly_seq_scheme.find1("asym_id"_key == strand1.front().asym_id() and "seq_id"_key == beg1SeqID);
-		auto e1 = pdbx_poly_seq_scheme.find1("asym_id"_key == strand1.front().asym_id() and "seq_id"_key == end1SeqID);
-		auto b2 = pdbx_poly_seq_scheme.find1("asym_id"_key == strand2.front().asym_id() and "seq_id"_key == beg2SeqID);
-		auto e2 = pdbx_poly_seq_scheme.find1("asym_id"_key == strand2.front().asym_id() and "seq_id"_key == end2SeqID);
-
-		if (not(b1 and e1 and b2 and e2))
-		{
-			if (cif::VERBOSE > 0)
-				std::cerr << "error looking up strand ends" << std::endl;
-			continue;
-		}
-
-		pdbx_struct_sheet_hbond.emplace({
-			{ "sheet_id", cif::cif_id_for_number(sheet) },
-			{ "range_id_1", s1 + 1 },
-			{ "range_id_2", s2 + 1 },
-
-			{ "range_1_label_atom_id", beg1AtomID },
-			{ "range_1_label_comp_id", b1["mon_id"].as<std::string>() },
-			{ "range_1_label_asym_id", b1["asym_id"].as<std::string>() },
-			{ "range_1_label_seq_id", b1["seq_id"].as<std::string>() },
-			{ "range_1_PDB_ins_code", b1["pdb_ins_code"].as<std::string>() },
-			{ "range_1_auth_atom_id", beg1AtomID },
-			{ "range_1_auth_comp_id", b1["auth_mon_id"].as<std::string>() },
-			{ "range_1_auth_asym_id", b1["pdb_strand_id"].as<std::string>() },
-			{ "range_1_auth_seq_id", b1["auth_seq_num"].as<std::string>() },
-			{ "range_2_label_atom_id", beg2AtomID },
-			{ "range_2_label_comp_id", b2["mon_id"].as<std::string>() },
-			{ "range_2_label_asym_id", b2["asym_id"].as<std::string>() },
-			{ "range_2_label_seq_id", b2["seq_id"].as<std::string>() },
-			{ "range_2_PDB_ins_code", b2["pdb_ins_code"].as<std::string>() },
-			{ "range_2_auth_atom_id", beg2AtomID },
-			{ "range_2_auth_comp_id", b2["auth_mon_id"].as<std::string>() },
-			{ "range_2_auth_asym_id", b2["pdb_strand_id"].as<std::string>() },
-			{ "range_2_auth_seq_id", b2["auth_seq_num"].as<std::string>() },
-		});
-	}
-
 	auto &struct_sheet_range = db["struct_sheet_range"];
 
-	for (const auto &[key, iSheet] : sheetMap)
+	for (int sheetNr : sheetNrs)
 	{
-		for (auto &&[sheet, strand] : strands)
+		auto sheetID = cif::cif_id_for_number(sheetNr - 1);
+
+		struct_sheet.emplace({
+			{ "id", sheetID },
+			{ "number_strands",
+				std::count_if(strands.begin(), strands.end(), [nr = sheetNr](std::tuple<std::tuple<int,int>, res_list> const &s)
+				{
+					const auto &[strandID, strand] = s;
+					return strand.front().sheet() == nr;
+				})
+			}
+		});
+
+		for (auto &&[strandTuple, strand] : strands)
 		{
-			if (sheet != iSheet)
+			if (strand.front().sheet() != sheetNr)
 				continue;
+			
+			std::string strandID = cif::cif_id_for_number(strand.front().strand() - 1);
 
 			std::sort(strand.begin(), strand.end(), [](dssp::residue_info const &a, dssp::residue_info const &b)
 				{ return a.nr() < b.nr(); });
@@ -718,8 +384,9 @@ void writeSheets(cif::datablock &db, const dssp &dssp)
 			auto &beg = strand.front();
 			auto &end = strand.back();
 
-			struct_sheet_range.emplace({ { "sheet_id", cif::cif_id_for_number(sheet) },
-				{ "id", strandNrForResidue(strand.front()) + 1 },
+			struct_sheet_range.emplace({
+				{ "sheet_id", sheetID },
+				{ "id", strandID },
 				{ "beg_label_comp_id", beg.compound_id() },
 				{ "beg_label_asym_id", beg.asym_id() },
 				{ "beg_label_seq_id", beg.seq_id() },
@@ -743,14 +410,21 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 	// Write out the DSSP ladders
 	struct ladder_info
 	{
-		ladder_info(const std::string &label, bool parallel, const dssp::residue_info &a, const dssp::residue_info &b)
-			: label(label)
+		ladder_info(int label, int sheet, bool parallel, const dssp::residue_info &a, const dssp::residue_info &b)
+			: ladder(label)
+			, sheet(sheet)
 			, parallel(parallel)
 			, pairs({ { a, b } })
 		{
 		}
 
-		std::string label;
+		bool operator<(const ladder_info &rhs) const
+		{
+			return ladder < rhs.ladder;
+		}
+
+		int ladder;
+		int sheet;
 		bool parallel;
 		std::vector<std::pair<dssp::residue_info, dssp::residue_info>> pairs;
 	};
@@ -766,12 +440,10 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 			if (not p)
 				continue;
 
-			auto label = cif::cif_id_for_number(ladder);
-
 			bool is_new = true;
 			for (auto &l : ladders)
 			{
-				if (l.label != label)
+				if (l.ladder != ladder)
 					continue;
 
 				assert(l.parallel == parallel);
@@ -791,10 +463,12 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 			if (not is_new)
 				continue;
 
-			ladders.emplace_back(label, parallel, res, p);
+			ladders.emplace_back(ladder, res.sheet() - 1, parallel, res, p);
 		}
 	}
 
+	std::sort(ladders.begin(), ladders.end());
+
 	auto &dssp_struct_ladder = db["dssp_struct_ladder"];
 
 	for (const auto &l : ladders)
@@ -802,7 +476,11 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 		const auto &[beg1, beg2] = l.pairs.front();
 		const auto &[end1, end2] = l.pairs.back();
 
-		dssp_struct_ladder.emplace({ { "id", l.label },
+		dssp_struct_ladder.emplace({
+			{ "id", cif::cif_id_for_number(l.ladder) },
+			{ "sheet_id", cif::cif_id_for_number(l.sheet) },
+			{ "range_id_1", cif::cif_id_for_number(beg1.strand() - 1) },
+			{ "range_id_2", cif::cif_id_for_number(beg2.strand() - 1) },
 			{ "type", l.parallel ? "parallel" : "anti-parallel" },
 
 			{ "beg_1_label_comp_id", beg1.compound_id() },
@@ -845,14 +523,15 @@ void writeStatistics(cif::datablock &db, const dssp &dssp)
 
 	auto &dssp_statistics = db["dssp_statistics"];
 
-	dssp_statistics.emplace({ { "entry_id", db.name() },
+	auto stats_i = dssp_statistics.emplace({ { "entry_id", db.name() },
 		{ "nr_of_residues", stats.count.residues },
 		{ "nr_of_chains", stats.count.chains },
 		{ "nr_of_ss_bridges_total", stats.count.SS_bridges },
 		{ "nr_of_ss_bridges_intra_chain", stats.count.intra_chain_SS_bridges },
-		{ "nr_of_ss_bridges_inter_chain", stats.count.SS_bridges - stats.count.intra_chain_SS_bridges },
-
-		{ "accessible_surface_of_protein", stats.accessible_surface } });
+		{ "nr_of_ss_bridges_inter_chain", stats.count.SS_bridges - stats.count.intra_chain_SS_bridges } });
+	
+	if (stats.accessible_surface > 0)
+		(*stats_i)["accessible_surface_of_protein"] = stats.accessible_surface;
 
 	auto &dssp_struct_hbonds = db["dssp_statistics_hbond"];
 
@@ -881,7 +560,7 @@ void writeStatistics(cif::datablock &db, const dssp &dssp)
 
 	using histogram_data_type = std::tuple<const char *, const uint32_t *>;
 	for (const auto &[type, values] : std::vector<histogram_data_type>{
-			 { "parallel_bridges_per_ladder", stats.histogram.residues_per_alpha_helix },
+			 { "residues_per_alpha_helix", stats.histogram.residues_per_alpha_helix },
 			 { "parallel_bridges_per_ladder", stats.histogram.parallel_bridges_per_ladder },
 			 { "antiparallel_bridges_per_ladder", stats.histogram.antiparallel_bridges_per_ladder },
 			 { "ladders_per_sheet", stats.histogram.ladders_per_sheet } })
@@ -925,10 +604,20 @@ void writeStatistics(cif::datablock &db, const dssp &dssp)
 
 void writeSummary(cif::datablock &db, const dssp &dssp)
 {
+	bool writeAccessibility = dssp.get_statistics().accessible_surface > 0;
+
 	// A approximation of the old format
 
 	auto &dssp_struct_summary = db["dssp_struct_summary"];
 
+	// prime the category with the field labels we need, this is to ensure proper order in writing out the data.
+
+	for (auto label : { "entry_id", "label_comp_id", "label_asym_id", "label_seq_id", "secondary_structure",
+			"ss_bridge", "helix_3_10", "helix_alpha", "helix_pi", "helix_pp", "bend", "chirality", "sheet",
+			"strand", "ladder_1", "ladder_2", "accessibility", "TCO", "kappa", "alpha", "phi", "psi",
+			"x_ca", "y_ca", "z_ca"})
+		dssp_struct_summary.add_column(label);
+
 	for (auto res : dssp)
 	{
 
@@ -978,10 +667,9 @@ void writeSummary(cif::datablock &db, const dssp &dssp)
 		if (res.bend())
 			bend = "S";
 
-		double alpha = res.alpha();
 		std::string chirality = ".";
-		if (alpha != 360)
-			chirality = alpha < 0 ? "-" : "+";
+		if (res.alpha().has_value())
+			chirality = *res.alpha() < 0 ? "-" : "+";
 
 		std::string ladders[2] = { ".", "." };
 
@@ -996,7 +684,7 @@ void writeSummary(cif::datablock &db, const dssp &dssp)
 
 		auto const &[cax, cay, caz] = res.ca_location();
 
-		dssp_struct_summary.emplace({
+		cif::row_initializer data{
 			{ "entry_id", db.name() },
 			{ "label_comp_id", res.compound_id() },
 			{ "label_asym_id", res.asym_id() },
@@ -1014,30 +702,60 @@ void writeSummary(cif::datablock &db, const dssp &dssp)
 			{ "bend", bend },
 			{ "chirality", chirality },
 
+			{ "sheet", res.sheet() ? cif::cif_id_for_number(res.sheet() - 1) : "." },
+			{ "strand", res.strand() ? cif::cif_id_for_number(res.strand() - 1) : "." },
 			{ "ladder_1", ladders[0] },
 			{ "ladder_2", ladders[1] },
 
-			{ "sheet", res.sheet() ? cif::cif_id_for_number(res.sheet() - 1) : "." },
-
-			{ "accesssibility", res.accessibility(), 1 },
-
-			{ "TCO", res.tco(), 3 },
-			{ "kappa", res.kappa(), 1 },
-			{ "alpha", res.alpha(), 1 },
-			{ "phi", res.phi(), 1 },
-			{ "psi", res.psi(), 1 },
-
 			{ "x_ca", cax, 1 },
 			{ "y_ca", cay, 1 },
 			{ "z_ca", caz, 1 },
-		});
+		};
+
+		if (writeAccessibility)
+			data.emplace_back("accessibility", res.accessibility(), 1);
+
+		if (res.tco().has_value())
+			data.emplace_back("TCO", *res.tco(), 3);
+		else
+			data.emplace_back("TCO", ".");
+
+		if (res.kappa().has_value())
+			data.emplace_back("kappa", *res.kappa(), 1);
+		else
+			data.emplace_back("kappa", ".");
+
+		if (res.alpha().has_value())
+			data.emplace_back("alpha", *res.alpha(), 1);
+		else
+			data.emplace_back("alpha", ".");
+
+		if (res.phi().has_value())
+			data.emplace_back("phi", *res.phi(), 1);
+		else
+			data.emplace_back("phi", ".");
+
+		if (res.psi().has_value())
+			data.emplace_back("psi", *res.psi(), 1);
+		else
+			data.emplace_back("psi", ".");
+		
+		dssp_struct_summary.emplace(std::move(data));
 	}
 }
 
-void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool writeExperimental, std::ostream &os)
+void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool writeExperimental)
 {
 	using namespace std::literals;
 
+	auto &validator = const_cast<cif::validator &>(*db.get_validator());
+	if (validator.get_validator_for_category("dssp_struct_summary") == nullptr)
+	{
+		auto dssp_extension = cif::load_resource("dssp-extension.dic");
+		if (dssp_extension)
+			cif::extend_dictionary(validator, *dssp_extension);
+	}
+
 	if (dssp.empty())
 	{
 		if (cif::VERBOSE > 0)
@@ -1140,10 +858,6 @@ void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool wr
 					{ "end_auth_comp_id", re.compound_id() },
 					{ "end_auth_asym_id", re.auth_asym_id() },
 					{ "end_auth_seq_id", re.auth_seq_id() }
-
-					// { "pdbx_PDB_helix_class", vS(39, 40) },
-				    // { "details", vS(41, 70) },
-				    // { "pdbx_PDB_helix_length", vI(72, 76) }
 				});
 
 				st = t;
@@ -1166,6 +880,4 @@ void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool wr
 		{ "version", kVersionNumber },
 		{ "date", kBuildDate },
 		{ "classification", "model annotation" } });
-
-	db.write(os);
 }
