@@ -28,6 +28,8 @@
 
 #include "dssp.hpp"
 
+#include "dssp-io.hpp"
+
 #include <deque>
 #include <iomanip>
 #include <numeric>
@@ -249,13 +251,16 @@ const float
 
 struct dssp::residue
 {
+	residue(residue &&) = default;
+	residue &operator=(residue &&) = default;
+
 	residue(int model_nr,
-		std::string_view pdb_strand_id, int pdb_seq_num, std::string_view pdb_ins_code,
-		const std::vector<cif::row_handle> &atoms)
+		std::string_view pdb_strand_id, int pdb_seq_num, std::string_view pdb_ins_code)
 		: mPDBStrandID(pdb_strand_id)
 		, mPDBSeqNum(pdb_seq_num)
 		, mPDBInsCode(pdb_ins_code)
 		, mChainBreak(chain_break_type::None)
+		, m_model_nr(model_nr)
 	{
 		// update the box containing all atoms
 		mBox[0].mX = mBox[0].mY = mBox[0].mZ = std::numeric_limits<float>::max();
@@ -263,105 +268,105 @@ struct dssp::residue
 
 		mH = point{ std::numeric_limits<float>::max(), std::numeric_limits<float>::max(), std::numeric_limits<float>::max() };
 
-		int seen = 0;
-
-		std::array<point, 4> chiralAtoms;
-		for (auto &p : chiralAtoms)
+		for (auto &p : m_chiralAtoms)
 			p = {};
+	}
 
-		for (auto atom : atoms)
+	void addAtom(cif::row_handle atom)
+	{
+		std::string asymID, compID, atomID, type, authAsymID;
+		std::optional<std::string> altID;
+		int seqID, authSeqID;
+		std::optional<int> model;
+		float x, y, z;
+
+		cif::tie(asymID, compID, atomID, altID, type, seqID, model, x, y, z, authAsymID, authSeqID) =
+			atom.get("label_asym_id", "label_comp_id", "label_atom_id", "label_alt_id", "type_symbol", "label_seq_id",
+				"pdbx_PDB_model_num", "Cartn_x", "Cartn_y", "Cartn_z",
+				"auth_asym_id", "auth_seq_id");
+
+		if (model and model != m_model_nr)
+			return;
+
+		if (m_seen == 0)
 		{
-			std::string asymID, compID, atomID, type, authAsymID;
-			std::optional<std::string> altID;
-			int seqID, authSeqID;
-			std::optional<int> model;
-			float x, y, z;
+			mAsymID = asymID;
+			mCompoundID = compID;
+			mSeqID = seqID;
 
-			cif::tie(asymID, compID, atomID, altID, type, seqID, model, x, y, z, authAsymID, authSeqID) =
-				atom.get("label_asym_id", "label_comp_id", "label_atom_id", "label_alt_id", "type_symbol", "label_seq_id",
-					"pdbx_PDB_model_num", "Cartn_x", "Cartn_y", "Cartn_z",
-					"auth_asym_id", "auth_seq_id");
+			mAuthSeqID = authSeqID;
+			mAuthAsymID = authAsymID;
 
-			if (model and model != model_nr)
-				continue;
+			mType = MapResidue(mCompoundID);
 
-			if (seen == 0)
-			{
-				mAsymID = asymID;
-				mCompoundID = compID;
-				mSeqID = seqID;
-
-				mAuthSeqID = authSeqID;
-				mAuthAsymID = authAsymID;
-
-				mType = MapResidue(mCompoundID);
-
-				if (altID)
-					mAltID = *altID;
-			}
-
-			if (atomID == "CA")
-			{
-				seen |= 1;
-				mCAlpha = { x, y, z };
-				ExtendBox(mCAlpha, kRadiusCA + 2 * kRadiusWater);
-
-				if (mType == kValine)
-					chiralAtoms[1] = mCAlpha;
-			}
-			else if (atomID == "C")
-			{
-				seen |= 2;
-				mC = { x, y, z };
-				ExtendBox(mC, kRadiusC + 2 * kRadiusWater);
-			}
-			else if (atomID == "N")
-			{
-				seen |= 4;
-				mH = mN = { x, y, z };
-				ExtendBox(mN, kRadiusN + 2 * kRadiusWater);
-			}
-			else if (atomID == "O")
-			{
-				seen |= 8;
-				mO = { x, y, z };
-				ExtendBox(mO, kRadiusO + 2 * kRadiusWater);
-			}
-			else if (type != "H")
-			{
-				seen |= 16;
-				mSideChain.emplace_back(atomID, point{ x, y, z });
-				ExtendBox(point{ x, y, z }, kRadiusSideAtom + 2 * kRadiusWater);
-
-				if (mType == kLeucine)
-				{
-					if (atomID == "CG")
-						chiralAtoms[0] = { x, y, z };
-					else if (atomID == "CB")
-						chiralAtoms[1] = { x, y, z };
-					else if (atomID == "CD1")
-						chiralAtoms[2] = { x, y, z };
-					else if (atomID == "CD2")
-						chiralAtoms[3] = { x, y, z };
-				}
-				else if (mType == kValine)
-				{
-					if (atomID == "CB")
-						chiralAtoms[0] = { x, y, z };
-					else if (atomID == "CG1")
-						chiralAtoms[2] = { x, y, z };
-					else if (atomID == "CG2")
-						chiralAtoms[3] = { x, y, z };
-				}
-			}
+			if (altID)
+				mAltID = *altID;
 		}
 
+		if (atomID == "CA")
+		{
+			m_seen |= 1;
+			mCAlpha = { x, y, z };
+			ExtendBox(mCAlpha, kRadiusCA + 2 * kRadiusWater);
+
+			if (mType == kValine)
+				m_chiralAtoms[1] = mCAlpha;
+		}
+		else if (atomID == "C")
+		{
+			m_seen |= 2;
+			mC = { x, y, z };
+			ExtendBox(mC, kRadiusC + 2 * kRadiusWater);
+		}
+		else if (atomID == "N")
+		{
+			m_seen |= 4;
+			mH = mN = { x, y, z };
+			ExtendBox(mN, kRadiusN + 2 * kRadiusWater);
+		}
+		else if (atomID == "O")
+		{
+			m_seen |= 8;
+			mO = { x, y, z };
+			ExtendBox(mO, kRadiusO + 2 * kRadiusWater);
+		}
+		else if (type != "H")
+		{
+			m_seen |= 16;
+			mSideChain.emplace_back(atomID, point{ x, y, z });
+			ExtendBox(point{ x, y, z }, kRadiusSideAtom + 2 * kRadiusWater);
+
+			if (mType == kLeucine)
+			{
+				if (atomID == "CG")
+					m_chiralAtoms[0] = { x, y, z };
+				else if (atomID == "CB")
+					m_chiralAtoms[1] = { x, y, z };
+				else if (atomID == "CD1")
+					m_chiralAtoms[2] = { x, y, z };
+				else if (atomID == "CD2")
+					m_chiralAtoms[3] = { x, y, z };
+			}
+			else if (mType == kValine)
+			{
+				if (atomID == "CB")
+					m_chiralAtoms[0] = { x, y, z };
+				else if (atomID == "CG1")
+					m_chiralAtoms[2] = { x, y, z };
+				else if (atomID == "CG2")
+					m_chiralAtoms[3] = { x, y, z };
+			}
+		}
+	}
+
+	void finish()
+	{
 		const int kSeenAll = (1 bitor 2 bitor 4 bitor 8);
-		mComplete = (seen bitand kSeenAll) == kSeenAll;
+		mComplete = (m_seen bitand kSeenAll) == kSeenAll;
 
 		if (mType == kValine or mType == kLeucine)
-			mChiralVolume = dot_product(chiralAtoms[1] - chiralAtoms[0],
-				cross_product(chiralAtoms[2] - chiralAtoms[0], chiralAtoms[3] - chiralAtoms[0]));
+			mChiralVolume = dot_product(m_chiralAtoms[1] - m_chiralAtoms[0],
+				cross_product(m_chiralAtoms[2] - m_chiralAtoms[0], m_chiralAtoms[3] - m_chiralAtoms[0]));
 
 		mRadius = mBox[1].mX - mBox[0].mX;
 		if (mRadius < mBox[1].mY - mBox[0].mY)
@@ -413,6 +418,9 @@ struct dssp::residue
 	void SetSheet(uint32_t inSheet) { mSheet = inSheet; }
 	uint32_t GetSheet() const { return mSheet; }
 
+	void SetStrand(uint32_t inStrand) { mStrand = inStrand; }
+	uint32_t GetStrand() const { return mStrand; }
+
 	bool IsBend() const { return mBend; }
 	void SetBend(bool inBend) { mBend = inBend; }
 
@@ -531,7 +539,8 @@ struct dssp::residue
 	float mAccessibility = 0;
 	float mChiralVolume = 0;
 
-	float mAlpha = 360, mKappa = 360, mPhi = 360, mPsi = 360, mTCO = 0, mOmega = 360;
+	// float mAlpha = 360, mKappa = 360, mPhi = 360, mPsi = 360, mTCO = 0, mOmega = 360;
+	std::optional<float> mAlpha, mKappa, mPhi, mPsi, mTCO, mOmega;
 
 	residue_type mType;
 	uint8_t mSSBridgeNr = 0;
@@ -539,9 +548,13 @@ struct dssp::residue
 	HBond mHBondDonor[2] = {}, mHBondAcceptor[2] = {};
 	bridge_partner mBetaPartner[2] = {};
 	uint32_t mSheet = 0;
+	uint32_t mStrand = 0;	// Added to ease the writing of mmCIF's struct_sheet and friends
 	helix_position_type mHelixFlags[4] = { helix_position_type::None, helix_position_type::None, helix_position_type::None, helix_position_type::None }; //
 	bool mBend = false;
 	chain_break_type mChainBreak = chain_break_type::None;
+
+	int m_seen = 0, m_model_nr = 0;
+	std::array<point, 4> m_chiralAtoms;
 };
 
 // --------------------------------------------------------------------
@@ -679,7 +692,7 @@ float residue::CalculateSurface(const std::vector<residue> &inResidues)
 		point center = r.mCenter;
 		float radius = r.mRadius;
 
-		if (distance(mCenter, center) < mRadius + radius)
+		if (distance_sq(mCenter, center) < (mRadius + radius) * (mRadius + radius))
 			neighbours.push_back(const_cast<residue *>(&r));
 	}
 
@@ -758,24 +771,23 @@ double CalculateHBondEnergy(residue &inDonor, residue &inAcceptor)
 
 // --------------------------------------------------------------------
 
-void CalculateHBondEnergies(std::vector<residue> &inResidues)
+void CalculateHBondEnergies(std::vector<residue> &inResidues, std::vector<std::tuple<uint32_t, uint32_t>> &q)
 {
-	// Calculate the HBond energies
-	for (uint32_t i = 0; i + 1 < inResidues.size(); ++i)
+	std::unique_ptr<cif::progress_bar> progress;
+	if (cif::VERBOSE == 0 or cif::VERBOSE == 1)
+		progress.reset(new cif::progress_bar(q.size(), "calculate hbond energies"));
+
+	for (const auto &[i, j] : q)
 	{
 		auto &ri = inResidues[i];
+		auto &rj = inResidues[j];
 
-		for (uint32_t j = i + 1; j < inResidues.size(); ++j)
-		{
-			auto &rj = inResidues[j];
-
-			if (distance(ri.mCAlpha, rj.mCAlpha) < kMinimalCADistance)
-			{
-				CalculateHBondEnergy(ri, rj);
-				if (j != i + 1)
-					CalculateHBondEnergy(rj, ri);
-			}
-		}
+		CalculateHBondEnergy(ri, rj);
+		if (j != i + 1)
+			CalculateHBondEnergy(rj, ri);
+		
+		if (progress)
+			progress->consumed(1);
 	}
 }
 
@@ -851,58 +863,64 @@ bool Linked(const bridge &a, const bridge &b)
 
 // --------------------------------------------------------------------
 
-void CalculateBetaSheets(std::vector<residue> &inResidues, statistics &stats)
+void CalculateBetaSheets(std::vector<residue> &inResidues, statistics &stats, std::vector<std::tuple<uint32_t, uint32_t>> &q)
 {
+	// if (cif::VERBOSE)
+	// 	std::cerr << "calculating beta sheets" << std::endl;
+
+	std::unique_ptr<cif::progress_bar> progress;
+	if (cif::VERBOSE == 0 or cif::VERBOSE == 1)
+		progress.reset(new cif::progress_bar(q.size(), "calculate beta sheets"));
+
 	// Calculate Bridges
 	std::vector<bridge> bridges;
 
-	for (uint32_t i = 1; i + 4 < inResidues.size(); ++i)
+	for (const auto &[i, j] : q)
 	{
+		if (progress)
+			progress->consumed(1);
+
 		auto &ri = inResidues[i];
+		auto &rj = inResidues[j];
 
-		for (uint32_t j = i + 3; j + 1 < inResidues.size(); ++j)
+		bridge_type type = TestBridge(ri, rj);
+		if (type == bridge_type::None)
+			continue;
+
+		bool found = false;
+		for (bridge &bridge : bridges)
 		{
-			auto &rj = inResidues[j];
-
-			bridge_type type = TestBridge(ri, rj);
-			if (type == bridge_type::None)
+			if (type != bridge.type or i != bridge.i.back() + 1)
 				continue;
 
-			bool found = false;
-			for (bridge &bridge : bridges)
+			if (type == bridge_type::Parallel and bridge.j.back() + 1 == j)
 			{
-				if (type != bridge.type or i != bridge.i.back() + 1)
-					continue;
-
-				if (type == bridge_type::Parallel and bridge.j.back() + 1 == j)
-				{
-					bridge.i.push_back(i);
-					bridge.j.push_back(j);
-					found = true;
-					break;
-				}
-
-				if (type == bridge_type::AntiParallel and bridge.j.front() - 1 == j)
-				{
-					bridge.i.push_back(i);
-					bridge.j.push_front(j);
-					found = true;
-					break;
-				}
-			}
-
-			if (not found)
-			{
-				bridge bridge = {};
-
-				bridge.type = type;
 				bridge.i.push_back(i);
-				bridge.chainI = ri.mAsymID;
 				bridge.j.push_back(j);
-				bridge.chainJ = rj.mAsymID;
-
-				bridges.push_back(bridge);
+				found = true;
+				break;
 			}
+
+			if (type == bridge_type::AntiParallel and bridge.j.front() - 1 == j)
+			{
+				bridge.i.push_back(i);
+				bridge.j.push_front(j);
+				found = true;
+				break;
+			}
+		}
+
+		if (not found)
+		{
+			bridge bridge = {};
+
+			bridge.type = type;
+			bridge.i.push_back(i);
+			bridge.chainI = ri.mAsymID;
+			bridge.j.push_back(j);
+			bridge.chainJ = rj.mAsymID;
+
+			bridges.push_back(bridge);
 		}
 	}
 
@@ -1082,12 +1100,35 @@ void CalculateBetaSheets(std::vector<residue> &inResidues, statistics &stats)
 			inResidues[i].SetSheet(bridge.sheet);
 		}
 	}
+
+	// Create 'strands'. A strand is a range of residues without a gap in between
+	// that belong to the same sheet.
+
+	int strand = 0;
+	for (uint32_t iSheet = 1; iSheet < sheet; ++iSheet)
+	{
+		int lastNr = -1;
+		for (auto &res : inResidues)
+		{
+			if (res.mSheet != iSheet)
+				continue;
+			
+			if (lastNr + 1 < res.mNumber)
+				++strand;
+			
+			res.mStrand = strand;
+			lastNr = res.mNumber;
+		}
+	}
 }
 
 // --------------------------------------------------------------------
 
 void CalculateAlphaHelices(std::vector<residue> &inResidues, statistics &stats, bool inPreferPiHelices = true)
 {
+	if (cif::VERBOSE)
+		std::cerr << "calculating alpha helices" << std::endl;
+
 	// Helix and Turn
 	for (helix_type helixType : { helix_type::_3_10, helix_type::alpha, helix_type::pi })
 	{
@@ -1114,8 +1155,8 @@ void CalculateAlphaHelices(std::vector<residue> &inResidues, statistics &stats,
 
 	for (auto &r : inResidues)
 	{
-		double kappa = r.mKappa;
-		r.SetBend(kappa != 360 and kappa > 70);
+		if (r.mKappa.has_value())
+			r.SetBend(*r.mKappa > 70);
 	}
 
 	for (uint32_t i = 1; i + 4 < inResidues.size(); ++i)
@@ -1179,7 +1220,7 @@ void CalculateAlphaHelices(std::vector<residue> &inResidues, statistics &stats,
 
 	std::string asym;
 	size_t helixLength = 0;
-	for (auto r : inResidues)
+	for (auto &r : inResidues)
 	{
 		if (r.mAsymID != asym)
 		{
@@ -1204,6 +1245,9 @@ void CalculateAlphaHelices(std::vector<residue> &inResidues, statistics &stats,
 
 void CalculatePPHelices(std::vector<residue> &inResidues, statistics &stats, int stretch_length)
 {
+	if (cif::VERBOSE)
+		std::cerr << "calculating pp helices" << std::endl;
+
 	size_t N = inResidues.size();
 
 	const float epsilon = 29;
@@ -1216,8 +1260,8 @@ void CalculatePPHelices(std::vector<residue> &inResidues, statistics &stats, int
 
 	for (uint32_t i = 1; i + 1 < inResidues.size(); ++i)
 	{
-		phi[i] = static_cast<float>(inResidues[i].mPhi);
-		psi[i] = static_cast<float>(inResidues[i].mPsi);
+		phi[i] = static_cast<float>(inResidues[i].mPhi.value_or(360));
+		psi[i] = static_cast<float>(inResidues[i].mPsi.value_or(360));
 	}
 
 	for (uint32_t i = 1; i + 3 < inResidues.size(); ++i)
@@ -1367,41 +1411,65 @@ DSSP_impl::DSSP_impl(const cif::datablock &db, int model_nr, int min_poly_prolin
 	: mDB(db)
 	, m_min_poly_proline_stretch_length(min_poly_proline_stretch_length)
 {
+	using namespace cif::literals;
+
+	if (cif::VERBOSE)
+		std::cerr << "loading residues" << std::endl;
+
 	int resNumber = 0;
 
 	auto &pdbx_poly_seq_scheme = mDB["pdbx_poly_seq_scheme"];
 	auto &atom_site = mDB["atom_site"];
 
-	for (auto r : pdbx_poly_seq_scheme)
+	using key_type = std::tuple<std::string,int>;
+	using index_type = std::map<key_type, size_t>;
+
+	index_type index;
+
+	mResidues.reserve(pdbx_poly_seq_scheme.size());
+
+	for (const auto &[asym_id, seq_id, pdb_strand_id, pdb_seq_num, pdb_ins_code]
+		: pdbx_poly_seq_scheme.rows<std::string,int, std::string, int, std::string>("asym_id", "seq_id", "pdb_strand_id", "pdb_seq_num", "pdb_ins_code"))
 	{
-		std::string pdb_strand_id, pdb_ins_code;
-		int pdb_seq_num;
+		index[{asym_id, seq_id}] = mResidues.size();
+		mResidues.emplace_back(model_nr, pdb_strand_id, pdb_seq_num, pdb_ins_code);
+	}
 
-		cif::tie(pdb_strand_id, pdb_seq_num, pdb_ins_code) = r.get("pdb_strand_id", "pdb_seq_num", "pdb_ins_code");
+	for (auto atom : atom_site)
+	{
+		std::string asym_id;
+		int seq_id;
 
-		chain_break_type brk = chain_break_type::NewChain;
-
-		residue residue(model_nr, pdb_strand_id, pdb_seq_num, pdb_ins_code, pdbx_poly_seq_scheme.get_children(r, atom_site));
-
-		if (not residue.mComplete)
+		cif::tie(asym_id, seq_id) = atom.get("label_asym_id", "label_seq_id");
+		auto i = index.find({asym_id, seq_id});
+		if (i == index.end())
 			continue;
+		
+		mResidues[i->second].addAtom(atom);
+	}
 
+	for (auto &residue : mResidues)
+		residue.finish();
+	
+	mResidues.erase(std::remove_if(mResidues.begin(), mResidues.end(), [](const dssp::residue &r) { return not r.mComplete; }), mResidues.end());
+	mStats.count.chains = 1;
+
+	chain_break_type brk = chain_break_type::NewChain;
+	for (size_t i = 0; i < mResidues.size(); ++i)
+	{
+		auto &residue = mResidues[i];
 		++resNumber;
 
-		if (mResidues.empty())
-			mStats.count.chains = 1;
-		else
+		if (i > 0)
 		{
-			if (mResidues.back().mAsymID != residue.mAsymID)
-				++mStats.count.chains;
-
-			if (distance(mResidues.back().mC, residue.mN) > kMaxPeptideBondLength)
+			if (distance(mResidues[i - 1].mC, mResidues[i].mN) > kMaxPeptideBondLength)
 			{
-				if (mResidues.back().mAsymID == residue.mAsymID)
-				{
-					++mStats.count.chains;
+				++mStats.count.chains;
+				if (mResidues[i - 1].mAsymID == mResidues[i].mAsymID)
 					brk = chain_break_type::Gap;
-				}
+				else
+					brk = chain_break_type::NewChain;
+
 				++resNumber;
 			}
 		}
@@ -1409,8 +1477,6 @@ DSSP_impl::DSSP_impl(const cif::datablock &db, int model_nr, int min_poly_prolin
 		residue.mChainBreak = brk;
 		residue.mNumber = resNumber;
 
-		mResidues.emplace_back(std::move(residue));
-
 		brk = chain_break_type::None;
 	}
 
@@ -1442,7 +1508,10 @@ DSSP_impl::DSSP_impl(const cif::datablock &db, int model_nr, int min_poly_prolin
 					nextNext.mCAlpha,
 					cur.mCAlpha);
 				float skap = std::sqrt(1 - ckap * ckap);
-				cur.mKappa = std::atan2(skap, ckap) * static_cast<float>(180 / kPI);
+
+				auto kappa = std::atan2(skap, ckap) * static_cast<float>(180 / kPI);
+				if (not std::isnan(kappa))
+					cur.mKappa = kappa;
 			}
 		}
 
@@ -1481,6 +1550,9 @@ DSSP_impl::DSSP_impl(const cif::datablock &db, int model_nr, int min_poly_prolin
 
 void DSSP_impl::calculateSecondaryStructure()
 {
+	if (cif::VERBOSE)
+		std::cerr << "calculating secondary structure" << std::endl;
+
 	using namespace cif::literals;
 
 	for (auto [asym1, seq1, asym2, seq2] : mDB["struct_conn"].find<std::string, int, std::string, int>("conn_type_id"_key == "disulf",
@@ -1507,8 +1579,45 @@ void DSSP_impl::calculateSecondaryStructure()
 		mSSBonds.emplace_back(&*r1, &*r2);
 	}
 
-	CalculateHBondEnergies(mResidues);
-	CalculateBetaSheets(mResidues, mStats);
+	// Prefetch the c-alpha positions. No, really, that might be the trick
+
+	std::vector<point> cAlphas;
+	cAlphas.reserve(mResidues.size());
+	for (auto &r : mResidues)
+		cAlphas.emplace_back(r.mCAlpha);
+
+	std::unique_ptr<cif::progress_bar> progress;
+	if (cif::VERBOSE == 0 or cif::VERBOSE == 1)
+		progress.reset(new cif::progress_bar((mResidues.size() * (mResidues.size() - 1)) / 2, "calculate distances"));
+
+	// Calculate the HBond energies
+	std::vector<std::tuple<uint32_t,uint32_t>> near;
+
+	for (uint32_t i = 0; i + 1 < mResidues.size(); ++i)
+	{
+		auto cai = cAlphas[i];
+
+		for (uint32_t j = i + 1; j < mResidues.size(); ++j)
+		{
+			auto caj = cAlphas[j];
+
+			if (distance_sq(cai, caj) > (kMinimalCADistance * kMinimalCADistance))
+				continue;
+
+			near.emplace_back(i, j);
+		}
+
+		if (progress)
+			progress->consumed(mResidues.size() - i - 1);
+	}
+
+	if (cif::VERBOSE > 0)
+		std::cerr << "Considering " << near.size() << " pairs of residues" << std::endl;
+
+	progress.reset(nullptr);
+
+	CalculateHBondEnergies(mResidues, near);
+	CalculateBetaSheets(mResidues, mStats, near);
 	CalculateAlphaHelices(mResidues, mStats);
 	CalculatePPHelices(mResidues, mStats, m_min_poly_proline_stretch_length);
 
@@ -1888,32 +1997,32 @@ std::string dssp::residue_info::pdb_ins_code() const
 	return m_impl->mPDBInsCode;
 }
 
-float dssp::residue_info::alpha() const
+std::optional<float> dssp::residue_info::alpha() const
 {
 	return m_impl->mAlpha;
 }
 
-float dssp::residue_info::kappa() const
+std::optional<float> dssp::residue_info::kappa() const
 {
 	return m_impl->mKappa;
 }
 
-float dssp::residue_info::omega() const
+std::optional<float> dssp::residue_info::omega() const
 {
 	return m_impl->mOmega;
 }
 
-float dssp::residue_info::phi() const
+std::optional<float> dssp::residue_info::phi() const
 {
 	return m_impl->mPhi;
 }
 
-float dssp::residue_info::psi() const
+std::optional<float> dssp::residue_info::psi() const
 {
 	return m_impl->mPsi;
 }
 
-float dssp::residue_info::tco() const
+std::optional<float> dssp::residue_info::tco() const
 {
 	return m_impl->mTCO;
 }
@@ -2053,6 +2162,11 @@ int dssp::residue_info::sheet() const
 	return m_impl->GetSheet();
 }
 
+int dssp::residue_info::strand() const
+{
+	return m_impl->GetStrand();
+}
+
 std::tuple<dssp::residue_info, double> dssp::residue_info::acceptor(int i) const
 {
 	auto &a = m_impl->mHBondAcceptor[i];
@@ -2159,3 +2273,17 @@ std::string dssp::get_pdb_header_line(pdb_record_type pdb_record) const
 			return {};
 	}
 }
+
+// --------------------------------------------------------------------
+
+void dssp::write_legacy_output(std::ostream& os) const
+{
+	writeDSSP(*this, os);
+}
+
+void dssp::annotate(cif::datablock &db, bool writeOther, bool writeDSSPCategories) const
+{
+	annotateDSSP(db, *this, writeOther, writeDSSPCategories);
+}
+
+
